#!/usr/bin/env python
'''
$Id$
Copyright (c) 2007, Meethune Bhowmick <meethune@gmail.com>

Permission to use, copy, modify, and distribute this software for any
purpose with or without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
'''
import getopt, sys
from getpass import getpass
from os.path import basename
try:
    from otp import OTP
except ImportError:
    print "Please install python-otp from http://labix.org/python-otp"
    sys.exit(1)

def usage():
    ''' Print simple usage message '''
    arguments = '''Generate an one time password (RFC2289)
Options:
    -h, --help 
    -n, --number [number of passwords to generate] (optional: default 1)
    -H, --hash [md4|md5|sha1] (optional: default sha1)
    -s, --sequence [sequence number] (mandatory)
    -S, --seed [seed string] (mandatory)
    '''
    
    print "Usage: " + basename(sys.argv[0]),
    print "[-n passwords] [-H md4|md5|sha1] [-s sequence] [-S seed]"
    print arguments

def die(exitval = 0, msg=None):
    ''' Print usage, a message if available, then exits program '''
    usage()
    if msg:
        print msg
    sys.exit(exitval)

def get_passwd():
    ''' Return user's S/Key password '''
    return getpass("Enter your S/Key password: ")
    
def get_challenge(options):
    ''' Return One time password for user '''
    challenge = []
    iterations = options['iterations']
    sequence = options['sequence']
    passwd = get_passwd()
    otp = OTP(format="words", hash=options['hash'])
    while iterations != 0:
        skey_passwd = otp.generate(passwd, sequence, options['seed'])[0]
        challenge.append((sequence, skey_passwd))
        iterations -= 1
        sequence -= 1
    return challenge

def display_results(challenge):
    ''' Print results. Append sequence number if more than one iteration '''
    if len(challenge) > 1:
        for each in challenge:
            print str(each[0]) + ' => ' + each[1]
    else:
        print challenge[0][1]

def get_options_dict(option, arguements):
    ''' Take cli option and put them into dictionary for parsing '''
    arguements = arguements.strip('=')
    if option in ("-h", "--help"):
        die()
    if option in ("-n", "--numbers"):
        if arguements.isdigit():
            return ('iterations', int(arguements))
        else:
            die(2)
    if option in ("-H" "--hash"):
        if arguements not in ("md4", "md5", "sha1"):
            die(2)
        else:
            return ('hash', arguements)
    if option in ("-s", "--sequence"):
        if arguements.isdigit():
            return ('sequence', int(arguements))
        else:
            die(2)
    if option in ("-S", "--seed"):
        return ('seed', arguements)

    return False

def parse_opts():
    ''' Parse command line arguments '''
    opt_dict = {
                'iterations': 1,
                'hash': 'sha1',
                'sequence': None,
                'seed': None 
               }
    try:
        opts = getopt.getopt(sys.argv[1:], \
                "hn:H:s:S:", ["help","number=","hash=","sequence=","seed="])[0]
        # -h --help 
        # -n --number [number of iterations] (optional: default 1)
        # -H --hash [md4|md5|sha1] (optional: default sha1)
        # -s --sequence [sequence number] (mandatory)
        # -S --seed [seed string] (mandatory)
    except getopt.GetoptError:
        die(2)
    for option, arguements in opts:
        (key, val) = get_options_dict(option, arguements)
        opt_dict[key] = val

    # Make sure mandatory values have been set
    if None in opt_dict.values():
        usage()
        sys.exit(2)
    # Error out if we want to generate more passwords than is available
    if opt_dict['iterations'] > opt_dict['sequence']:
        msg = "There are not enough sequences left to run "
        msg += str(opt_dict['iterations']) + " iterations."
        die(2, msg)
    
    return opt_dict

def main():
    ''' Launch Program '''
    options = parse_opts()
    challenge_results = get_challenge(options)
    display_results(challenge_results)
    return

if __name__ == "__main__":
    main()
    sys.exit()

# vim:syntax=python
# vim:sw=4:softtabstop=4:expandtab
